SELECT * FROM sys.dm_os_sys_info
SELECT * FROM sys.dm_os_nodes
SELECT * FROM sys.dm_os_schedulers
SELECT * FROM sys.dm_os_workers
SELECT * FROM sys.dm_os_tasks
SELECT * FROM sys.dm_os_threads


-- SQL Server 2005, forces the display of ALL wait types in sys.dm_os_wait_stats
DBCC TRACEON (8001, -1)
DBCC TRACEOFF(8001, -1)

SELECT * FROM sys.dm_os_wait_stats
SELECT TOP 10 * FROM sys.dm_os_wait_stats ORDER BY wait_time_ms DESC

SELECT * FROM sys.dm_os_waiting_tasks
SELECT * FROM sys.dm_os_latch_stats
SELECT * FROM sys.dm_os_spinlock_stats

SELECT * FROM sys.dm_io_virtual_file_stats(null, null)
SELECT * FROM sys.dm_io_pending_io_requests

DBCC SQLPERF('waitstats')
DBCC SQLPERF('umsstats')

SELECT * FROM sys.dm_os_wait_stats
DBCC SQLPERF('sys.dm_os_wait_stats', CLEAR)

SELECT * FROM sys.dm_os_latch_stats
DBCC SQLPERF('sys.dm_os_latch_stats', CLEAR)

SELECT * FROM sys.dm_os_memory_node_access_stats
DBCC SQLPERF('sys.dm_os_memory_node_access_stats', CLEAR)

SELECT	*, (wait_time_ms - signal_wait_time_ms) AS resource_wait_time_ms 
FROM	sys.dm_os_wait_stats

SELECT	database_id,
		DB_NAME(database_id) AS database_name,
		file_id, 
		-- sample_ms,
		-- reads
		num_of_reads,
		num_of_bytes_read,
		num_of_bytes_read / CASE WHEN num_of_reads = 0 THEN 1 ELSE num_of_reads END AS avg_bytes_read,
		io_stall_read_ms,
		io_stall_read_ms / CASE WHEN num_of_reads = 0 THEN 1 ELSE num_of_reads END AS avg_io_stall_read,
		-- writes
		num_of_writes,
		num_of_bytes_written,
		num_of_bytes_written / CASE WHEN num_of_writes = 0 THEN 1 ELSE num_of_writes END AS avg_bytes_written,
		io_stall_write_ms,
		io_stall_write_ms / CASE WHEN num_of_writes = 0 THEN 1 ELSE num_of_writes END AS avg_io_stall_write,
		-- iops
		num_of_reads + num_of_writes AS num_iops,
		(num_of_bytes_read + num_of_bytes_written) AS num_of_bytes_iops,
		(num_of_bytes_read + num_of_bytes_written) / CASE WHEN num_of_reads + num_of_writes = 0 THEN 1 ELSE (num_of_reads + num_of_writes) END AS avg_bytes_iops,
		io_stall_read_ms + io_stall_write_ms AS io_stall_iops,
		io_stall,
		io_stall / CASE WHEN num_of_reads + num_of_writes = 0 THEN 1 ELSE (num_of_reads + num_of_writes) END AS avg_io_stall,
		
		((num_of_reads + num_of_writes) / CASE WHEN num_of_writes = 0 THEN 1 ELSE num_of_writes END) * 100 AS percentage_reads,
		((num_of_writes + num_of_reads)/ CASE WHEN num_of_reads = 0 THEN 1 ELSE num_of_reads END) * 100 AS percentage_writes,
		(num_of_bytes_read / CASE WHEN num_of_bytes_written = 0 THEN 1 ELSE num_of_bytes_written END) * 100 AS percentage_bytes_read,
		(num_of_bytes_written / CASE WHEN num_of_bytes_read = 0 THEN 1 ELSE num_of_bytes_read END) * 100 AS percentage_bytes_written
	
		-- , size_on_disk_bytes
		-- , file_handle

FROM	sys.dm_io_virtual_file_stats(NULL, NULL)

-- session level wait states using X/Events

IF EXISTS(SELECT * FROM sys.server_event_sessions WHERE name='Waits')
	DROP EVENT SESSION Waits ON SERVER
GO

CREATE EVENT SESSION Waits 
ON SERVER
ADD EVENT sqlos.wait_info 
(
	ACTION	(sqlserver.plan_handle, sqlserver.session_id)
	WHERE	(
			package0.equal_uint64(opcode,1) 
	AND		package0.not_equal_uint64(wait_type, 120) 
	AND		sqlserver.session_id > 50)
			),
ADD EVENT sqlos.wait_info_external 
(
	ACTION	(sqlserver.plan_handle, sqlserver.session_id)
	WHERE	(
			package0.equal_uint64(opcode,1) 
	AND		package0.not_equal_uint64(wait_type, 120) 
	AND		sqlserver.session_id > 50)
			)
ADD TARGET package0.ring_buffer 
(	
	SET max_memory= 1024
)
WITH 
(
	max_dispatch_latency = 1 seconds
)
GO

ALTER EVENT SESSION Waits ON SERVER
STATE = start
GO

USE [AdventureWorks];
GO

DBCC DROPCLEANBUFFERS
GO

SELECT	pvt.[SalesPersonID]
		,pvt.[FullName]
		,pvt.[Title]
		,pvt.[SalesTerritory]
		,pvt.[2002]
		,pvt.[2003]
		,pvt.[2004] 
FROM	(	SELECT	soh.[SalesPersonID]
					,c.[FirstName] + ' ' + COALESCE(c.[MiddleName], '') + ' ' + c.[LastName] AS [FullName]
					,e.[Title]
					,st.[Name] AS [SalesTerritory]
					,soh.[SubTotal]
					,YEAR(DATEADD(m, 6, soh.[OrderDate])) AS [FiscalYear] 
			FROM	[Sales].[SalesPerson] sp 
			INNER	JOIN [Sales].[SalesOrderHeader] soh ON sp.[SalesPersonID] = soh.[SalesPersonID]
			INNER	JOIN [Sales].[SalesTerritory] st ON sp.[TerritoryID] = st.[TerritoryID] 
			INNER	JOIN [HumanResources].[Employee] e  ON soh.[SalesPersonID] = e.[EmployeeID] 
			INNER	JOIN [Person].[Contact] c ON e.[ContactID] = c.ContactID 
		) AS soh 
PIVOT 
(
    SUM([SubTotal]) 
    FOR [FiscalYear] 
    IN ([2002], [2003], [2004])
) AS pvt;
GO

-- wait information is gathered
-- disable event collection
--
ALTER EVENT SESSION Waits ON SERVER
DROP EVENT sqlos.wait_info,
DROP EVENT sqlos.wait_info_external
GO

IF (OBJECT_ID('tempdb..#waits') IS NOT NULL)
BEGIN
	DROP TABLE #waits
END

SELECT	objWaits.value('(action/value)[2]', 'int') AS session_id,
		objWaits.value('(data/text)[1]', 'nvarchar(1024)') AS wait_type,
		objWaits.value('(data/value)[3]', 'int') AS duration,
		objWaits.value('(data/value)[6]', 'int') AS signal_duration,
		cast(objWaits.value('(action/value)[1]', 'nvarchar(1024)') AS xml) AS plan_handle
INTO	#waits
FROM	(	SELECT	CAST(xest.target_data AS xml)
					waits
			FROM	sys.dm_xe_session_targets AS xest
			JOIN	sys.dm_xe_sessions xes ON xes.address = xest.event_session_address
			WHERE	xest.target_name = 'ring_buffer' 
			AND		xes.name = 'Waits'
		) AS allwaits
CROSS APPLY waits.nodes('//event') as T(objWaits)

SELECT	wait_type, 
		total_wait_time = SUM(duration), 
		total_signal_time = SUM(signal_duration) 
FROM	#waits 
WHERE	session_id = @@spid
GROUP	BY wait_type
ORDER	BY 2 DESC

SELECT	* 
FROM	#waits 
WHERE	session_id = @@spid
GO

IF EXISTS(SELECT * FROM sys.server_event_sessions WHERE name = N'Waits')
	DROP EVENT SESSION Waits ON SERVER
GO

CREATE EVENT SESSION Waits ON SERVER
ADD EVENT sqlos.wait_info 
(
	ACTION	(sqlserver.plan_handle, sqlserver.session_id)
	WHERE	(
			package0.equal_uint64(opcode,1) 
	AND		package0.not_equal_uint64(wait_type, 120) 
	AND		sqlserver.session_id > 50
			)
),
ADD EVENT sqlos.wait_info_external 
(
	ACTION	(sqlserver.plan_handle, sqlserver.session_id)
	WHERE	(
			package0.equal_uint64(opcode,1) 
	AND		package0.not_equal_uint64(wait_type, 120) 
	AND		sqlserver.session_id > 50)
			)
ADD TARGET package0.ring_buffer 
( 
	SET		max_memory = 1024
)
WITH 
(
	max_dispatch_latency = 1 seconds
)

ALTER EVENT SESSION Waits ON SERVER
STATE = start
GO

SELECT	pvt.[SalesPersonID]
		,pvt.[FullName]
		,pvt.[Title]
		,pvt.[SalesTerritory]
		,pvt.[2002]
		,pvt.[2003]
		,pvt.[2004] 
FROM	(	SELECT	soh.[SalesPersonID]
					,c.[FirstName] + ' ' + COALESCE(c.[MiddleName], '') + ' ' + c.[LastName] AS [FullName]
					,e.[Title]
					,st.[Name] AS [SalesTerritory]
					,soh.[SubTotal]
					,YEAR(DATEADD(m, 6, soh.[OrderDate])) AS [FiscalYear] 
			FROM	[Sales].[SalesPerson] sp 
			INNER	JOIN [Sales].[SalesOrderHeader] soh ON sp.[SalesPersonID] = soh.[SalesPersonID]
			INNER	JOIN [Sales].[SalesTerritory] st ON sp.[TerritoryID] = st.[TerritoryID] 
			INNER	JOIN [HumanResources].[Employee] e  ON soh.[SalesPersonID] = e.[EmployeeID] 
			INNER	JOIN [Person].[Contact] c ON e.[ContactID] = c.ContactID 
		) AS soh 
PIVOT 
(
    SUM([SubTotal]) 
    FOR [FiscalYear] 
    IN ([2002], [2003], [2004])
) AS pvt;
GO

IF (OBJECT_ID('tempdb..#waits2') IS NOT NULL)
BEGIN
	DROP TABLE #waits2
END

SELECT	objWaits.value('(action/value)[2]', 'int') AS session_id,
		objWaits.value('(data/text)[1]', 'nvarchar(1024)') AS wait_type,
		objWaits.value('(data/value)[3]', 'int') AS duration,
		objWaits.value('(data/value)[6]', 'int') AS signal_duration,
		cast(objWaits.value('(action/value)[1]', 'nvarchar(1024)') AS xml) AS plan_handle
into	#waits_2
FROM	(	SELECT	CAST(xest.target_data AS xml)
					waits
			FROM	sys.dm_xe_session_targets AS xest
			join	sys.dm_xe_sessions AS xes ON xes.address = xest.event_session_address
			WHERE	xest.target_name = 'ring_buffer' 
			AND		xes.name = 'Waits'
) AS allwaits
CROSS APPLY waits.nodes('//event') AS T(objWaits)

SELECT	wait_type, 
		total_wait_time = SUM(duration), 
		total_signal_time = SUM(signal_duration) 
FROM	#waits 
WHERE	session_id = @@spid
GROUP	BY wait_type
ORDER	BY 2 DESC

SELECT	wait_type, 
		total_wait_time = SUM(duration), 
		total_signal_time = SUM(signal_duration) 
FROM	#waits_2 
WHERE	session_id = @@spid
GROUP	BY wait_type
ORDER	BY 2 DESC

SELECT	* 
FROM	#waits_2 
WHERE	session_id = @@spid
GO

IF EXISTS(SELECT * FROM sys.server_event_sessions WHERE name = N'Waits')
	DROP EVENT SESSION Waits ON SERVER
GO

