/*
Kevin Kline, 13-July-2105
More information online at https://technet.microsoft.com/en-us/library/dd576261(v=sql.100).aspx. 

sys.sysdesend  system dialog endpoint send: This table tracks each conversation 
  when messages are sent from the initiator database. There is one clustered 
  index on the conversation handle.
sys.sysdercv  system dialog endpoint receive: This table tracks conversations 
  used on the target database. There is one clustered index on the conversation 
  handle.
sys.sysxmitqueue  system transmission queue: This table shows all messages 
  currently in the transmission queue that have been sent but havent been 
  transmitted to the target yet. This system table should be monitored regularly 
  to make sure the queue is being processed and messages are not growing faster 
  than being processed. If this table grows too large, your database files may grow!
*/

-- SSB Transmission row count
SELECT p.rows [XmitQ Depth] 
FROM sys.objects as o
JOIN sys.partitions AS p ON p.object_id = o.object_id
WHERE o.name LIKE 'sysxmitqueue'

/*
Monitoring Service Broker activation tasks

Using Perfmon counters for the object SQLServer:Broker Activation (per instance):
  - Task Limit Reached/sec  Service Broker is attempting to start a new 
    activation task, but the maximum number of activation tasks that can run 
	concurrently has been reached, as configured by the queues 
	MAX_QUEUE_READERS parameter.
  - Tasks Running  The current number of activation tasks running.
  - Tasks Started/sec  The number of tasks started per second.

Using DMVs:
  - sys.dm_broker_activated_tasks joined to sys.dm_exec_requests on session_id
    shows currently running service Broker activation tasks.
*/
SELECT * --object_name, counter_name, cntr_value
FROM sys.dm_os_performance_counters
WHERE object_name = 'SQLServer:Broker Activation'
  AND counter_name IN ('Task Limit Reached/sec','Tasks Running','Tasks Started/sec')
  AND instance_name = '_Total';

/*
For raw throughput, track SQLServer:Broker Messages/Sec on the target.
*/
SELECT object_name, counter_name, [cntr_value (mb)] = cntr_value/1024.
FROM sys.dm_os_performance_counters
WHERE OBJECT_NAME LIKE 'SQLServer:Broker%'
  --AND counter_name IN ('Target Server Memory (KB)','Total Server Memory (KB)');