/*
A table or a partition can have three allocation units - IN_ROW_DATA, LOB_DATA 
and ROW_OVERFLOW_DATA. Usually, most of the data in the table is stored in the 
IN_ROW_DATA allocation unit. Depending upon the row size and the table options, 
some data can be stored outside the row in ROW_OVERFLOW_DATA or LOB_DATA 
allocation units. Use the following script to determine how much data is stored 
in each of the three allocation units.
*/

SELECT OBJECT_NAME(p.object_id) AS Object_Name
       , i.name AS Index_Name
       , ps.in_row_used_page_count AS IN_ROW_DATA
       , ps.row_overflow_used_page_count AS ROW_OVERFLOW_DATA
       , ps.lob_used_page_count AS LOB_DATA
FROM sys.dm_db_partition_stats ps
JOIN sys.partitions p ON ps.partition_id = p.partition_id
JOIN sys.indexes i ON p.index_id = i.index_id AND p.object_id = i.object_id
WHERE OBJECTPROPERTY (p.[object_id], 'IsUserTable') = 1
ORDER BY [IN_ROW_DATA] DESC;

/* Free space in database files */
SELECT
	a.file_id,
	LOGICAL_NAME = a.name,
	PHYSICAL_FILENAME = a.physical_name,
	FILEGROUP_NAME = b.name,
	FILE_SIZE_MB = CONVERT(DECIMAL(12,2),ROUND(a.size/128.000,2)),
	SPACE_USED_MB = CONVERT(DECIMAL(12,2),ROUND(FILEPROPERTY(a.name,'SpaceUsed')/128.000,2)),
	FREE_SPACE_MB = CONVERT(DECIMAL(12,2),ROUND((a.size-FILEPROPERTY(a.name,'SpaceUsed'))/128.000,2))
FROM sys.database_files a 
LEFT OUTER JOIN sys.data_spaces b ON a.data_space_id = b.data_space_id;

/* 
On a Page-Compressed Index, Verifying That Nonleaf Pages are Row-Compressed 
Basically, Use the following query on a page-compressed index (clustered or 
nonclustered) to verify that the leaf-level pages are page-compressed; but 
the nonleaf pages are row-compressed, not page-compressed.
*/
SELECT
	o.name, ips.index_type_desc, p.partition_number, p.data_compression_desc,
	ips.index_level, ips.page_count, ips.compressed_page_count
FROM sys.dm_db_index_physical_stats
(DB_ID(), object_id(<insert index name here>), NULL, NULL, 'DETAILED') ips
JOIN sys.objects o ON o.object_id = ips.object_id
JOIN sys.partitions p ON p.object_id = o.object_id
ORDER BY ips.index_level;