/*

-- Last index statistics update

SELECT
   t.name AS Table_Name
   ,i.name AS Index_Name
   ,i.type_desc AS Index_Type
   ,STATS_DATE(i.object_id,i.index_id) AS Date_Updated
FROM
   sys.indexes i JOIN
   sys.tables t ON t.object_id = i.object_id
WHERE
   i.type > 0
ORDER BY
   t.name ASC
   ,i.type_desc ASC
   ,i.name ASC;
*/   
   
-- Create the table 
USE tempdb;
GO
SET NOCOUNT ON;
GO

IF EXISTS (SELECT * FROM sys.tables
            WHERE name = 'cause_page_splits')
    DROP TABLE cause_page_splits;
GO

CREATE TABLE cause_page_splits
(id    INT           IDENTITY(0,2) PRIMARY KEY,
 id2   BIGINT        DEFAULT 0,
 data1 VARCHAR(33)   NULL, 
 data2 VARCHAR(8000) NULL);
GO
 
-- 385 records of default size will fill the page 
-- This loop should create 100 +/- a few pages for GAM, IAM, etc.

DECLARE @counter1 tinyint
DECLARE @counter2 tinyint

SELECT @counter1 = 0, @counter2 = 0

WHILE @counter1 <> 38500
BEGIN
  INSERT INTO cause_page_splits DEFAULT VALUES;
END
GO

-- verify that there is only one data page 
DBCC IND(tempdb, cause_page_splits, -1);
GO

SET IDENTITY_INSERT cause_page_splits  ON;
GO
INSERT INTO cause_page_splits (id, id2, data1, data2)
      SELECT 111, 0, replicate('a', 33), replicate('b', 8000);
GO
SET IDENTITY_INSERT cause_page_splits  OFF;
GO

UPDATE cause_page_splits SET data1 = replicate('a', 33),
  data2 = replicate('b', 8000); 

SELECT * FROM cause_page_splits;
