-- DEMO: System information in internal values


-- Look at final column of DBCC PAGE output
USE pubs;
GO
BEGIN TRAN
  UPDATE titles
  SET price = price * 1.1
  WHERE title_id = 'PC9999';
  
-- rollback tran

---- Another connection

--SELECT * FROM dbo.titles
--  WHERE title_id = 'PC9999';

SELECT resource_type, resource_associated_entity_id, 
     resource_description, request_status, 
     request_mode,request_session_id, 
     resource_database_id 
    FROM sys.dm_tran_locks
   
-- Look at the page that is locked    
DBCC TRACEON(3604);
DBCC PAGE(pubs, 1, 286, 3);

-- Look at details of row in an index page
DBCC PAGE (credit, 3, 99, 3);



SELECT %%lockres%%, * FROM titles
--WHERE %%lockres%% = ' '

-- Why not always just search for %%lockres%%... 
-- Why should we ever have to look for locked resources using DBCC PAGE?

----------------------------------------------------------------

-- DEMO: system information in metadata

SELECT * FROM sys.dm_exec_query_optimizer_info
SELECT * FROM sys.dm_os_schedulers
SELECT * FROM sys.dm_os_sys_info

SELECT * FROM sys.system_internals_allocation_units
SELECT * FROM sys.system_internals_partitions
SELECT * FROM sys.system_internals_partition_columns

USE pubs
---- sys.system_internals_allocation_units
SELECT  OBJECT_NAME(OBJECT_ID), Rows, total_pages, first_page, root_page, first_iam_page
FROM sys.partitions p JOIN sys.system_internals_allocation_units a
	ON p.partition_id = a.container_id
WHERE OBJECT_NAME(OBJECT_ID) = 'authors'
	AND index_id  < 2;

-- Example: 0xB10000000100	-- use the value you get for first_page

-- Separate into bytes, 2 digits each:  B1 00 00 00 01 00
-- Reverse bytes: 00 01 00 00 00 B1
-- First two bytes are file number: 00 01 = File 1
-- Last four bytes are page number: 00 00 00 B1 = Page 177


IF EXISTS (SELECT * FROM sys.tables 
              WHERE name = 'nu_ci')
        DROP TABLE nu_ci;
GO

CREATE table nu_ci (col1 smallint, col2 tinyint);
CREATE clustered index c11 on nu_ci(col1);
CREATE index i1 on nu_ci (col2);

SELECT object_name(p.object_id) AS object_name, index_id, 
   partition_column_id AS index_column_id, 
   max_inrow_length, is_uniqueifier
FROM  sys.system_internals_partition_columns pc
    JOIN sys.partitions p
       on P.partition_id = PC.partition_id
WHERE object_name(P.object_id) = 'nu_ci'

-- Not really undoc'd

-- Look to see the information stored about compression in 
--  sys.dm_db_index_physical_stats() and sys.dm_db_index_operational_stats()



USE AdventureWorks

IF EXISTS (SELECT * FROM sys.tables WHERE name = 'details')
   DROP TABLE details;
go

SELECT * INTO details FROM sales.SalesOrderDetail

-- Check values before compression

-- Last column is "compressed"_page_count"
SELECT * FROM sys.dm_db_index_physical_stats(DB_ID('AdventureWorks'), OBJECT_ID('details'), NULL, NULL, 'detailed')

-- Last two columns are "page_compression_attempt_count" and "page_compression_success_count"
SELECT * FROM sys.dm_db_index_operational_stats(DB_ID('AdventureWorks'), OBJECT_ID('details'), NULL, NULL)

ALTER TABLE details REBUILD WITH (DATA_COMPRESSION = page);
GO


-- Check values after compression

-- Last column is "compressed"_page_count"
SELECT * FROM sys.dm_db_index_physical_stats(DB_ID('AdventureWorks'), OBJECT_ID('details'), NULL, NULL, 'detailed')

-- Last two columns are "page_compression_attempt_count" and "page_compression_success_count"
SELECT * FROM sys.dm_db_index_operational_stats(DB_ID('AdventureWorks'), OBJECT_ID('details'), NULL, NULL)
