CREATE TABLE Deposits (
 deposit_nbr INT NOT NULL PRIMARY KEY,
 transaction_date DATE NOT NULL,
 amount DECIMAL(18, 2));

INSERT INTO Deposits 
(deposit_nbr, transaction_date, amount)
VALUES
(1, '2011-01-01', 100.00),
(2, '2011-01-02', 50.00),
(3, '2011-01-03', 50.00),
(4, '2011-02-01', 10.00),
(5, '2011-03-01', 20.00),
(6, '2011-03-02', 30.00),
(7, '2011-03-02', 10.00),
(8, '2011-04-01', 100.00),
(9, '2011-04-02', 150.00);

-- FIRST_VALUE and LAST_VALUE
SELECT deposit_nbr,
       transaction_date,
	   amount,
	   FORMAT(transaction_date, 'yyyy-MM') AS year_month,
	   FIRST_VALUE(amount) OVER(PARTITION BY FORMAT(transaction_date, 'yyyy-MM') 
	                       ORDER BY deposit_nbr
	                       ROWS BETWEEN UNBOUNDED PRECEDING
						            AND UNBOUNDED FOLLOWING) AS first,
	   LAST_VALUE(amount) OVER(PARTITION BY FORMAT(transaction_date, 'yyyy-MM') 
	                           ORDER BY deposit_nbr
	                           ROWS BETWEEN UNBOUNDED PRECEDING
					    	            AND UNBOUNDED FOLLOWING) AS last
FROM Deposits
ORDER BY deposit_nbr;

-- LAG and LEAD for previous and subsequent row access
SELECT deposit_nbr,
       transaction_date,
	   amount,
	   LAG(amount) OVER(ORDER BY deposit_nbr) AS prior,
	   LEAD(amount) OVER(ORDER BY deposit_nbr) AS next
FROM Deposits
ORDER BY deposit_nbr;

-- offset to previous row and default
SELECT deposit_nbr,
       transaction_date,
	   amount,
	   LAG(amount, 3, 0) OVER(ORDER BY deposit_nbr) AS prior
FROM Deposits
ORDER BY deposit_nbr;

WITH Summary AS (
SELECT FORMAT(transaction_date, 'yyyy-MM') AS year_month,
       SUM(amount) AS monthly_total
FROM Deposits
GROUP BY FORMAT(transaction_date, 'yyyy-MM'))
SELECT year_month,
       monthly_total,
	   LAG(monthly_total, 3, 0) OVER(ORDER BY year_month) AS three_months_ago,
	   LEAD(monthly_total, 3, 0) OVER(ORDER BY year_month) AS three_months_later
FROM Summary
ORDER BY year_month;

DROP TABLE Deposits;