CREATE TABLE Deposits (
 deposit_nbr INT NOT NULL PRIMARY KEY,
 transaction_date DATE NOT NULL,
 amount DECIMAL(18, 2));

INSERT INTO Deposits 
(deposit_nbr, transaction_date, amount)
VALUES
(1, '2011-01-01', 100.00),
(2, '2011-01-02', 50.00),
(3, '2011-01-03', 50.00),
(4, '2011-02-01', 10.00),
(5, '2011-03-01', 20.00),
(6, '2011-03-02', 30.00),
(7, '2011-03-02', 10.00),
(8, '2011-04-01', 100.00),
(9, '2011-04-02', 150.00);

-- running total
SELECT deposit_nbr,
       transaction_date,
	   amount,
	   SUM(amount) OVER(ORDER BY deposit_nbr
	                    ROWS BETWEEN UNBOUNDED PRECEDING
						         AND CURRENT ROW) AS running_total
FROM Deposits
ORDER BY deposit_nbr;

-- running total by partition
SELECT deposit_nbr,
       transaction_date,
	   amount,
	   FORMAT(transaction_date, 'yyyy-MM') AS year_month,
	   SUM(amount) OVER(PARTITION BY FORMAT(transaction_date, 'yyyy-MM') 
	                    ORDER BY deposit_nbr
	                    ROWS BETWEEN UNBOUNDED PRECEDING
						         AND CURRENT ROW) AS monthly_running_total
FROM Deposits
ORDER BY deposit_nbr;

-- total of current and preceding or current and following rows
SELECT deposit_nbr,
       transaction_date,
	   amount,
	   SUM(amount) OVER(ORDER BY deposit_nbr
	                    ROWS BETWEEN 1 PRECEDING
						         AND CURRENT ROW) AS prior_current,
	   SUM(amount) OVER(ORDER BY deposit_nbr
	                    ROWS BETWEEN CURRENT ROW
						         AND 1 FOLLOWING) AS current_next
FROM Deposits
ORDER BY deposit_nbr;

-- difference betweeen ROWS and RANGE
SELECT deposit_nbr,
       transaction_date,
	   amount,
	   SUM(amount) OVER(ORDER BY transaction_date
	                    ROWS BETWEEN UNBOUNDED PRECEDING
						         AND CURRENT ROW) AS running_total_rows,
	   SUM(amount) OVER(ORDER BY transaction_date
	                    RANGE BETWEEN UNBOUNDED PRECEDING
						         AND CURRENT ROW) AS running_total_range
FROM Deposits
ORDER BY deposit_nbr;

SELECT deposit_nbr,
       transaction_date,
	   amount,
	   SUM(amount) OVER(ORDER BY transaction_date
	                    ROWS BETWEEN UNBOUNDED PRECEDING
						         AND CURRENT ROW) AS running_total,
	   SUM(amount) OVER(ORDER BY transaction_date
	                    ROWS BETWEEN CURRENT ROW
						         AND UNBOUNDED FOLLOWING) AS reverse_running_total
FROM Deposits
ORDER BY deposit_nbr;

-- moving average
SELECT deposit_nbr,
       transaction_date,
	   amount,
	   AVG(amount) OVER(ORDER BY deposit_nbr
	                    ROWS BETWEEN 2 PRECEDING
						         AND CURRENT ROW) AS moving_average
FROM Deposits
ORDER BY deposit_nbr;

DROP TABLE Deposits;