USE AdventureWorks2014;
GO
-- Return all rows sorted by the column DepartmentID.
SELECT DepartmentID, 
		Name, 
		GroupName
FROM	HumanResources.Department
ORDER BY 
		DepartmentID;
GO

-- Skip the first 5 rows from the sorted result set and return all remaining rows.
SELECT DepartmentID, 
		Name, 
		GroupName
FROM HumanResources.Department
ORDER BY 
		DepartmentID 
		OFFSET 5 ROWS;
GO

-- Skip 5 rows and return only the first 5 rows from the sorted result set.
SELECT DepartmentID, 
		Name, 
		GroupName
FROM	HumanResources.Department
ORDER BY 
		DepartmentID 
		OFFSET 5 ROWS
		FETCH NEXT 5 ROWS ONLY;
GO

-- Use variables to specify offset and fetch
DECLARE @Offset int = 5,
		@Fetch	int = 5 ;

SELECT DepartmentID, 
		Name, 
		GroupName
FROM	HumanResources.Department
ORDER BY 
		DepartmentID 
		OFFSET @Offset ROWS
		FETCH NEXT @Fetch ROWS ONLY;
GO


-- Use expressions to specify offset and fetch
DECLARE @Page int = 2,
		@PageSize int = 5

SELECT DepartmentID, 
		Name, 
		GroupName
FROM	HumanResources.Department
ORDER BY 
		DepartmentID 
		OFFSET (@Page - 1) * @PageSize ROWS
		FETCH NEXT @PageSize ROWS ONLY;
GO

-- Use subquery to specify offset and fetch
CREATE TABLE dbo.AppSettings (AppSettingID int NOT NULL, PageSize int NOT NULL);
GO
INSERT INTO dbo.AppSettings VALUES(1, 5);
GO

DECLARE @Page int = 2;
SELECT DepartmentID, 
		Name, 
		GroupName
FROM	HumanResources.Department
ORDER BY 
	DepartmentID ASC 
    OFFSET (@Page - 1) * (SELECT PageSize FROM dbo.AppSettings WHERE AppSettingID = 1) ROWS 
    FETCH NEXT (SELECT PageSize FROM dbo.AppSettings WHERE AppSettingID = 1) ROWS ONLY;
GO

DROP TABLE dbo.AppSettings ;
