CREATE TABLE Customers (
 customer_nbr INT NOT NULL PRIMARY KEY,
 customer_name VARCHAR(35) NOT NULL);

INSERT INTO Customers 
VALUES
(1, 'Joe'),
(2, 'John'),
(3, 'Jane'),
(4, 'Peter'),
(5, 'Mary'),
(6, 'Jose'),
(7, 'Daniel'),
(8, 'Adam'),
(9, 'Chris'),
(10, 'Tom'),
(11, 'Evan'),
(12, 'Lora');

SELECT customer_name
FROM Customers
ORDER BY customer_name;

/*

customer_name
---------------
Adam
Chris
Daniel
Evan
Jane
Joe
John
Jose
Lora
Mary
Peter
Tom

*/

DECLARE @page_nbr INT = 1, @page_size INT = 5;

-- using ROW_NUMBER and CTE
WITH Ranked ( customer_name, rn ) AS 
(
SELECT customer_name,
	   ROW_NUMBER() OVER (ORDER BY customer_name)
FROM Customers)
SELECT customer_name 
FROM Ranked
WHERE rn > ( @page_nbr - 1 ) * @page_size 
  AND rn <= @page_nbr * @page_size;

-- using OFFSET
-- first page
SELECT customer_name
FROM Customers
ORDER BY customer_name
OFFSET (@page_nbr - 1) * @page_size ROWS FETCH NEXT @page_size ROWS ONLY;

/*

customer_name
---------------
Adam
Chris
Daniel
Evan
Jane

*/

-- second page
SET @page_nbr = 2;

SELECT customer_name
FROM Customers
ORDER BY customer_name
OFFSET (@page_nbr - 1) * @page_size ROWS FETCH NEXT @page_size ROWS ONLY;

/*

customer_name
---------------
Joe
John
Jose
Lora
Mary

*/

-- last page
SELECT customer_name
FROM Customers
ORDER BY customer_name
OFFSET ((SELECT COUNT(*) FROM Customers) / @page_size) * @page_size 
ROWS FETCH NEXT @page_size ROWS ONLY;

/*

customer_name
---------------
Peter
Tom

*/

-- any 3 customers
SELECT customer_name
FROM Customers
ORDER BY (SELECT NULL)
OFFSET 0 ROWS FETCH NEXT 3 ROWS ONLY;

/*

customer_name
---------------
Joe
John
Jane

*/

-- skip 3 customers and return all others
SELECT customer_name
FROM Customers
ORDER BY customer_name
OFFSET 3 ROWS;

/*

customer_name
---------------
Evan
Jane
Joe
John
Jose
Lora
Mary
Peter
Tom

*/

-- fetch 0 rows
SELECT customer_name
FROM Customers
ORDER BY customer_name
OFFSET 0 ROWS FETCH FIRST (SELECT 0) ROWS ONLY;

/*

customer_name
---------------

(0 row(s) affected)

*/


-- top 3 customers
SELECT customer_name
FROM Customers
ORDER BY customer_name
OFFSET 0 ROWS FETCH NEXT 3 ROWS ONLY;

/*

customer_name
---------------
Adam
Chris
Daniel

*/

-- top 3 customers using TOP 
SELECT TOP(3) customer_name
FROM Customers
ORDER BY customer_name;

/*

customer_name
---------------
Adam
Chris
Daniel

*/

DROP TABLE Customers;
