DROP DATABASE SequenceTest
GO

CREATE DATABASE SequenceTest;
GO
USE SequenceTest;
GO
--Create the Test schema
CREATE SCHEMA Test ;
GO

-- Create a table
CREATE TABLE Test.Orders
    (OrderID int PRIMARY KEY,
    Name varchar(20) NOT NULL,
    Qty int NOT NULL);
GO

-- Create a sequence
CREATE SEQUENCE Test.CountBy1
    START WITH 1
    INCREMENT BY 1 ;
GO

-- Insert three records
INSERT Test.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Test.CountBy1, 'Tire', 2) ;
INSERT test.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Test.CountBy1, 'Seat', 1) ;
INSERT test.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Test.CountBy1, 'Brake', 1) ;
GO

-- View the table
SELECT * FROM Test.Orders ;
GO

CREATE SEQUENCE Test.CountBy5
    START WITH 10
    INCREMENT BY 5 ;
GO

INSERT Test.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Test.CountBy5, 'Spark Plug', 2) ;
INSERT test.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Test.CountBy5, 'Steering Wheel', 1) ;
INSERT test.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Test.CountBy5, 'Accelerator', 1) ;
GO

-- View the table
SELECT * FROM Test.Orders ;
GO


DECLARE @NextID int ;
SET @NextID = NEXT VALUE FOR Test.CountBy5;

SELECT @NextID;

-- Some work happens

INSERT Test.Orders (OrderID, Name, Qty)
    VALUES (@NextID, 'Rim', 2) ;

SELECT * FROM Test.Orders ;
GO

-- View the table
SELECT * FROM Test.Orders ;
GO

CREATE SEQUENCE Test.BucketID
    START WITH 1
    INCREMENT BY 1
	MINVALUE 1
	MAXVALUE 10
	CYCLE ;
GO

SELECT NEXT VALUE FOR Test.BucketID OVER (ORDER BY Name) AS BucketID, 
		ProductID, 
		Name, 
		ProductNumber 
FROM	AdventureWorks2012.Production.Product
WHERE	ProductNumber LIKE 'HN%'

ALTER SEQUENCE Test.BucketID
RESTART WITH 1 ;

SELECT NEXT VALUE FOR Test.BucketID OVER (ORDER BY Name) AS BucketID, 
		ProductID, 
		Name, 
		ProductNumber 
FROM	AdventureWorks2012.Production.Product
WHERE	ProductNumber LIKE 'HN%'

GO


SELECT * FROM sys.sequences



DECLARE @FirstSeqNum sql_variant,
		@LastSeqNum sql_variant,
		@CycleCount int,
		@SeqIncr sql_variant,
		@SeqMinVal sql_variant,
		@SeqMaxVal sql_variant ;

EXEC sys.sp_sequence_get_range
		@sequence_name = N'Test.CountBy1',
		@range_size = 1000,
		@range_first_value = @FirstSeqNum OUTPUT,
		@range_last_value = @LastSeqNum OUTPUT,
		@range_cycle_count = @CycleCount OUTPUT,
		@sequence_increment = @SeqIncr OUTPUT,
		@sequence_min_value = @SeqMinVal OUTPUT,
		@sequence_max_value = @SeqMaxVal OUTPUT ;

-- The following statement returns the output values
SELECT @FirstSeqNum AS FirstVal,
		@LastSeqNum AS LastVal,
		@CycleCount AS CycleCount,
		@SeqIncr AS SeqIncrement,
		@SeqMinVal AS MinSeq,
		@SeqMaxVal AS MaxSeq ;
