 
/*

- message_id not required to be existent in sys.messages
- THROW does not accept parameters for substitution (can use FORMATMESSAGE to work around)
- message_id must be INT >= 50000
- severity always 16 unless in CATCH to retrow error
- no WITH NOWAIT option
- honors XACT_ABORT
- acts like RETHROW preserving error number, severity, state, error text, line, etc.

*/

BEGIN TRY    
 
   -- inside code THROW is similar to RAISERROR with limitations
   THROW 51000, 'User error.', 1;

END TRY
 
BEGIN CATCH    

   -- inside CATCH rethrow the original error
   THROW;

END CATCH

/*

Msg 51000, Level 16, State 1, Line 16
User error.

*/

GO

BEGIN TRY    
 
   -- divide by zero error
   SELECT 1/0;

END TRY
 
BEGIN CATCH    

   -- inside CATCH rethrow the original error
   THROW;

END CATCH

/*

Msg 8134, Level 16, State 1, Line 5
Divide by zero error encountered.

*/

GO

CREATE TABLE Foo (
 keycol INT NOT NULL PRIMARY KEY,
 datacol CHAR(1) NULL);

GO

SET XACT_ABORT ON;

BEGIN TRAN

INSERT INTO Foo VALUES(1, 'a');
-- if error is raised the transaction will not roll back
RAISERROR('Some error', 16, 1);
INSERT INTO Foo VALUES(2, 'b');

COMMIT

GO

SELECT keycol, datacol FROM Foo;

GO

SET XACT_ABORT ON;

BEGIN TRAN

INSERT INTO Foo VALUES(3, 'c');
-- if error is raised the transaction is rolled back
THROW 51000, 'Another error', 1;
INSERT INTO Foo VALUES(4, 'd');

COMMIT

GO

SELECT keycol, datacol FROM Foo;

GO

DROP TABLE Foo;