/* Average stalls per read, write and total.
   Adding 1.0 to avoid division by zero errors. 
   The columns are arranged to provide summary info on the left,
   then more granular data further to the right.
   
   IO stalls are a good indication of disk IO pressure.*/
SELECT
  DB_NAME(database_id) AS "Database",
  FILE_NAME(file_id) AS "File",
    CAST( (io_stall_read_ms + io_stall_write_ms) /
    (1.0 + num_of_reads + num_of_writes) AS NUMERIC(10, 1))
    AS 'avg_io_stall_ms',
  (io_stall_read_ms + io_stall_write_ms) AS io_stalls,
  num_of_reads + num_of_writes AS total_io,
  io_stall_read_ms,
  io_stall_write_ms,
  num_of_reads,
  num_of_writes,
  CAST(io_stall_read_ms / (1.0 + num_of_reads) AS NUMERIC(10, 1))
    AS 'avg_read_stall_ms',
  CAST(io_stall_write_ms / (1.0 + num_of_writes) AS NUMERIC(10, 1))
    AS 'avg_write_stall_ms'
FROM sys.dm_io_virtual_file_stats(NULL, NULL)
ORDER BY avg_io_stall_ms DESC]];