USE msdb
GO

-- Selects the avg job duration for all active jobs & job steps
SELECT AVG(run_duration)
FROM dbo.sysjobhistory
WHERE run_status = 1


-- Selects any job & step ID that is 2x or longer than the AVG job run duration
--   for an active job
SELECT job_id, step_id
FROM msdb.dbo.sysjobhistory
WHERE run_duration >= 2 * (SELECT AVG(run_duration)
          FROM dbo.sysjobhistory
          WHERE run_status = 1)


-- Selects any job & step that is 2x or longer than the AVG job run duration
SELECT j.name, jh.step_name
FROM msdb.dbo.sysjobhistory AS jh
JOIN msdb.dbo.sysjobs AS j ON j.job_id = jh.job_id
WHERE run_duration >= 2 * (SELECT AVG(run_duration)
          FROM dbo.sysjobhistory)
  AND run_status = 1


-- Selects all jobs and steps that are twice or more as long as their AVG duration
SELECT j.name, jh.step_name, jh.run_date, jh.run_duration
FROM msdb.dbo.sysjobhistory AS jh
JOIN msdb.dbo.sysjobs AS j ON j.job_id = jh.job_id
WHERE jh.run_duration >= 2 * (SELECT AVG(sjh.run_duration)
          FROM sysjobhistory AS sjh
					WHERE sjh.job_id = jh.job_id
					  AND sjh.step_id = jh.step_id)
  AND jh.run_status = 1


-- Selects jobs and steps that are twice or more as long as their AVG duration
--   minus any jobs with miniscule run durations
SELECT j.name, jh.step_name, jh.run_date, jh.run_duration
FROM msdb.dbo.sysjobhistory AS jh
JOIN msdb.dbo.sysjobs AS j ON j.job_id = jh.job_id
WHERE jh.run_duration >= 2 * (SELECT AVG(sjh.run_duration)
          FROM sysjobhistory AS sjh
					WHERE sjh.job_id = jh.job_id
					  AND sjh.step_id = jh.step_id)
  AND jh.run_duration > 1 -- eliminates inconsequential run durations
	AND jh.run_status = 1 -- only active jobs
