/*
Transaction Related DMV
Server scope
*/

--Returns information about currently active lock manager resources. Each row represents a 
--currently active request to the lock manager for a lock that has been granted or is waiting to be granted.
SELECT 
	l.resource_type
	,l.resource_subtype
	,DB_NAME(l.resource_database_id) AS databaseScope
	,CASE
		WHEN resource_type = 'OBJECT' THEN OBJECT_NAME(l.resource_associated_entity_id)
		ELSE CONVERT(NVARCHAR(100), l.resource_associated_entity_id)
	END AS associatedEntity
	,l.resource_lock_partition
	,l.request_type
	,l.request_status
	,l.request_reference_count AS numberOfRequests
	,l.request_session_id
	,t.transaction_descriptor
	,l.request_owner_type
	,l.lock_owner_address
	,wt.wait_duration_ms
	,wt.wait_type
	,wt.blocking_session_id
	,wt.resource_description
	,r.status AS requestStatus
	,r.command AS requestCommandType
	,OBJECT_NAME(qt.objectid) AS callingObject
   ,SUBSTRING(qt.text,r.statement_start_offset / 2,( CASE WHEN r.statement_end_offset = -1 THEN LEN(CONVERT(NVARCHAR(MAX),qt.text)) * 2
                                                           ELSE r.statement_end_offset
                                                      END - r.statement_start_offset ) / 2) AS IndividualQueryText	
	,USER_NAME(r.[user_id]) AS userName
FROM 
	sys.dm_tran_locks AS l
	LEFT JOIN sys.dm_tran_session_transactions AS t ON l.request_session_id = t.session_id
	LEFT JOIN sys.dm_os_waiting_tasks AS wt ON l.lock_owner_address = wt.resource_address
	LEFT JOIN sys.dm_exec_requests AS r ON l.request_session_id = r.session_id
	CROSS APPLY sys.dm_exec_sql_text(r.sql_handle) AS qt
	;

--Returns information about transactions for the SQL Server instance.
SELECT 
	transaction_id
	,[name]
	,transaction_begin_time
	,CASE
		WHEN transaction_type = 1 THEN 'Read/write'
		WHEN transaction_type = 2 THEN 'Read-only'
		WHEN transaction_type = 3 THEN 'System'
		WHEN transaction_type = 4 THEN 'Distributed'
	END AS transactionType 
	,CASE
		WHEN transaction_state = 0 THEN 'IS NOT initialized'	
		WHEN transaction_state = 1 THEN 'Initialized, NOT started'
		WHEN transaction_state = 2 THEN 'Active'
		WHEN transaction_state = 3 THEN 'Ended, READ-only'
		WHEN transaction_state = 4 THEN 'COMMIT process initiated, DISTRIBUTED only'
		WHEN transaction_state = 5 THEN 'Prepared awaiting resolution'
		WHEN transaction_state = 6 THEN 'committed'
		WHEN transaction_state = 7 THEN 'being rolled back'
		WHEN transaction_state = 8 THEN 'rolled back'
	END AS transactionState
	,CASE
		WHEN dtc_state = 1 THEN 'Active'
		WHEN dtc_state = 2 THEN 'Prepared'
		WHEN dtc_state = 3 THEN 'Committed'
		WHEN dtc_state = 4 THEN 'Aborted'
		WHEN dtc_state = 5 THEN 'Recovered'
	END AS dtcState
FROM 
	sys.dm_tran_active_transactions;
	
--Returns information about transactions at the database level.	
SELECT 
	transaction_id
	,DB_NAME(database_id) AS dbName
	,database_transaction_begin_time AS startTime
	,CASE
		WHEN database_transaction_type = 1 THEN 'Read/write'
		WHEN database_transaction_type = 2 THEN 'Read-only'
		WHEN database_transaction_type = 3 THEN 'System'
	END AS transactionType
	,CASE
		WHEN database_transaction_state = 1 THEN 'The transaction has not been initialized.'
		WHEN database_transaction_state = 3 THEN 'The transaction has been initialized but has not generated any log records.'
		WHEN database_transaction_state = 4 THEN 'The transaction has generated log records.'
		WHEN database_transaction_state = 5 THEN 'The transaction has been prepared.'
		WHEN database_transaction_state = 10 THEN 'The transaction has been committed.'
		WHEN database_transaction_state = 11 THEN 'The transaction has been rolled back.'
		WHEN database_transaction_state = 12 THEN 'The transaction is being committed. In this state the log record is being generated, but it has not been materialized or persisted.'
	END AS transactionState
FROM 
	sys.dm_tran_database_transactions AS t
WHERE
	DB_NAME(database_id) NOT LIKE 'tempdb';