USE tempdb;
SET NOCOUNT ON;
go

-- Remove old version of demo database. Use brute force if necessary.
IF EXISTS (SELECT name FROM sys.databases WHERE name = N'AdventureWorksDW2012XL')
  BEGIN;
  ALTER DATABASE AdventureWorksDW2012XL SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
  DROP DATABASE AdventureWorksDW2012XL;
  END;
go

-- Create database by restoring from AdventureWorks2012 backup.
-- WARNING - change path names for your computer before running!!!
RESTORE DATABASE AdventureWorksDW2012XL
FROM DISK = N'C:\SQL 2012\Backup\AdventureWorksDW2012.bak'
WITH MOVE N'AdventureWorksDW2012_Data' TO N'D:\SQL 2012\Data\AdventureWorksDW2012XL_Data.mdf',
     MOVE N'AdventureWorksDW2012_Log'  TO N'C:\SQL 2012\Log\AdventureWorksDW2012XL_log.ldf',
     REPLACE, STATS = 10;
go

-- Ensure simple recovery - no need to have log backup hassle for a demo database.
ALTER DATABASE AdventureWorksDW2012XL SET RECOVERY SIMPLE;
go

-- Change file sizes - we're going to need a huge amount of disk space!
-- (The size used here should be enough, but I'll specify autogrow just in case).
ALTER DATABASE AdventureWorksDW2012XL
MODIFY FILE (NAME = N'AdventureWorksDW2012_Data', SIZE =   22GB, MAXSIZE = UNLIMITED, FILEGROWTH =  1GB);

ALTER DATABASE AdventureWorksDW2012XL
MODIFY FILE (NAME = N'AdventureWorksDW2012_Log',  SIZE = 1024MB, MAXSIZE = UNLIMITED, FILEGROWTH = 64MB);
go

-- If AdventureWorksDW2012XL was not created, the USE statement fails,
-- but the next batch will still execute.
-- Switch to tempdb first, so that we don't get test tables in other databases.
-- This needs to be in a separate batch, so it will execute even if the parse of the next USE fails.
USE tempdb;
go
USE AdventureWorksDW2012XL;
go


/*

SNIPPED:

Creation of FactResellerSalesXL table (and partitioning function and schema)
CLR function used for generating random numbers with specific distribution

*/

go

-- Generate a table of numbers for later use
CREATE TABLE dbo.Numbers
         (n int NOT NULL PRIMARY KEY);
-- Populate it with 100,000 numbers
INSERT INTO dbo.Numbers (n)
SELECT TOP(100000) ROW_NUMBER() OVER (ORDER BY ProductKey, DateKey)   -- Match smallest index in tablel
FROM   dbo.FactProductInventory;                                      -- Any table with >100,000 rows will do

-- We need to expand the DimDate table with 2011, 2012, and 2013.
WITH
Dates     AS (SELECT DATEADD(day, n, CAST('20101231' AS date)) AS NewDate
              FROM   dbo.Numbers),
DatesPlus AS (SELECT NewDate,
                     DATEPART(dd, NewDate) AS dd,
                     DATEPART(dy, NewDate) AS dy,
                     DATEPART(dw, NewDate) AS dw,
                     DATEPART(wk, NewDate) AS wk,
                     DATEPART(mm, NewDate) AS mm,
                     DATEPART(qq, NewDate) AS qq,
                     DATEPART(yy, NewDate) AS yy
              FROM   Dates
              WHERE  NewDate <= '20131231')
INSERT INTO dbo.DimDate
           (DateKey,
            FullDateAlternateKey,
            DayNumberOfWeek,
            EnglishDayNameOfWeek,
            SpanishDayNameOfWeek,
            FrenchDayNameOfWeek,
            DayNumberOfMonth,
            DayNumberOfYear,
            WeekNumberOfYear,
            EnglishMonthName,
            SpanishMonthName,
            FrenchMonthName,
            MonthNumberOfYear,
            CalendarQuarter,
            CalendarYear,
            CalendarSemester,
            FiscalQuarter,
            FiscalYear,
            FiscalSemester)
SELECT      10000 * yy + 100 * mm + dd,
            NewDate,
            dw,
            Weekdays.English,
            Weekdays.Spanish,
            Weekdays.French,
            dd,
            dy,
            wk,
            Months.English,
            Months.Spanish,
            Months.French,
            mm,
            qq,
            yy,
           (qq + 1) / 2,
            CASE WHEN qq <= 2 THEN qq + 2 ELSE qq - 2 END,
            CASE WHEN qq <= 2 THEN yy ELSE yy + 1 END,
           (6 - qq) / 2
FROM        DatesPlus
INNER JOIN (VALUES(1, 'Sunday', 'Domingo', 'Dimanche'),
                  (2, 'Monday', 'Lunes', 'Lundi'),
                  (3, 'Tuesday', 'Martes', 'Mardi'),
                  (4, 'Wednesday', 'Mircoles', 'Mercredi'),
                  (5, 'Thursday', 'Jueves', 'Jeudi'),
                  (6, 'Friday', 'Viernes', 'Vendredi'),
                  (7, 'Saturday', 'Sbado', 'Samedi')) AS Weekdays(DayNr, English, Spanish, French)
      ON    Weekdays.DayNr = DatesPlus.dw
INNER JOIN (VALUES(1 , 'January', 'Enero', 'Janvier'),
                  (2 , 'February', 'Febrero', 'Fvrier'),
                  (3 , 'March', 'Marzo', 'Mars'),
                  (4 , 'April', 'Abril', 'Avril'),
                  (5 , 'May', 'Mayo', 'Mai'),
                  (6 , 'June', 'Junio', 'Juin'),
                  (7 , 'July', 'Julio', 'Juillet'),
                  (8 , 'August', 'Agosto', 'Aot'),
                  (9 , 'September', 'Septiembre', 'Septembre'),
                  (10, 'October', 'Octubre', 'Octobre'),
                  (11, 'November', 'Noviembre', 'Novembre'),
                  (12, 'December', 'Diciembre', 'Dcembre')) AS Months(MonthNr, English, Spanish, French)
      ON    Months.MonthNr = DatesPlus.mm;


-- The table DimProduct is set up as a type-2 slowly changing dimension,
-- but with errors in the dates - and they're all in the past.
-- Let's change this to something more current (and correct).
-- Luckily, Microsoft uses only four distinct valid date ranges:
-- June 1, 1998 - current       --> unchanged
-- July 1, 2005 - June 30, 2002 --> Jan 1, 2009 - Dec 31, 2010
-- July 1, 2006 - June 30, 2003 --> Jan 1, 2011 - Dec 31, 2012
-- July 1, 2007 - current       --> Jan 1, 2013 - current
UPDATE dbo.DimProduct
SET    StartDate = CASE WHEN StartDate = '20050701' THEN '20090101'
                        WHEN StartDate = '20060701' THEN '20110101'
                           ELSE '20130101' END,
       EndDate   = CASE WHEN StartDate = '20050701' THEN '20101231'
                        WHEN StartDate = '20060701' THEN '20121231'
                        ELSE NULL END
WHERE  StartDate <> '19980601';
go


/*

SNIPPED:

Code to generate >100 million rows in FactResellerSalesXL

*/
